/*
 * Copyright 2017 Rockchip Electronics S.LSI Co. LTD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/

#ifndef __ROCKCHIP_RGA_H__
#define __ROCKCHIP_RGA_H__

#include <linux/videodev2.h>

typedef enum _RgaRotate {
    RGA_ROTATE_NONE = 0,
    RGA_ROTATE_90,
    RGA_ROTATE_180,
    RGA_ROTATE_270,
    RGA_ROTATE_VFLIP,       // Vertical Mirror
    RGA_ROTATE_HFLIP,       // Horizontal Mirror
} RgaRotate;

struct _RockchipRga;
typedef struct _RgaOps {
    /* Init rga contex */
    void (*initCtx)(struct _RockchipRga *rga);

    void (*setRotate)(struct _RockchipRga *rga, RgaRotate rotate);
    /* Set solid fill color, color:
     * Blue:  0xffff0000
     * Green: 0xff00ff00
     * Red:   0xff0000ff
     */
    void (*setFillColor)(struct _RockchipRga *rga, int color);

    /* Set source format */
    void (*setSrcFormat)(struct _RockchipRga *rga, __u32 v4l2Format,
		        __u32 width, __u32 height);

    /* Set destination format */
    void (*setDstFormat)(struct _RockchipRga *rga, __u32 v4l2Format,
		        __u32 width, __u32 height);

    /* Set source crop */
    void (*setSrcCrop)(struct _RockchipRga *rga, __u32 cropX, __u32 cropY,
                   __u32 cropW, __u32 cropH);

    /* Set destination crop */
    void (*setDstCrop)(struct _RockchipRga *rga, __u32 cropX, __u32 cropY,
                   __u32 cropW, __u32 cropH);

    /* Set source dma buffer file description */
    void (*setSrcBufferFd)(struct _RockchipRga *rga, int dmaFd);

    /* Set destination dma buffer file description */
    void (*setDstBufferFd)(struct _RockchipRga *rga, int dmaFd);

    /* Set source userspace data pointer */
    void (*setSrcBufferPtr)(struct _RockchipRga *rga, void *ptr);

    /* Set destination userspace data pointer */
    void (*setDstBufferPtr)(struct _RockchipRga *rga, void *ptr);

    /* Set source physical address, which allocated by RockchipCmaAlloc */
    void (*setSrcBufferPhys)(struct _RockchipRga *rga, unsigned long phys);

    /* Set destination physical address, which allocated by RockchipCmaAlloc */
    void (*setDstBufferPhys)(struct _RockchipRga *rga, unsigned long phys);

    int (*go)(struct _RockchipRga *rga);
} RgaOps;

typedef struct _RockchipRga {
    RgaOps *ops;
    unsigned long driver_data;
} RockchipRga;

RockchipRga * RgaCreate(void);
void RgaDestroy(RockchipRga *rga);

#endif
