#include <functional>
#include <iostream>
#include <pthread.h>
#include <sys/syscall.h>
#include <getopt.h>
#include <chrono>
#include <asm/types.h>
#include <assert.h>
#include <fcntl.h>
#include <dirent.h>
#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include <linux/stddef.h>
#include <linux/videodev2.h>
#include <errno.h>

#include <opencv2/opencv.hpp>

#include "rockchip_rga.hpp"

using namespace cv;
using namespace std;

const string winname = "Rga Convert";

unsigned long long nanoTime()
{
    struct timespec t;

    t.tv_sec = t.tv_nsec = 0;
    clock_gettime(CLOCK_MONOTONIC, &t);

    return (unsigned long long)(t.tv_sec) * 1000000000 + t.tv_nsec;
}

static void usage(char **argv)
{
    printf(
        "Usage: %s [Options]\n\n"
        "Options:\n"
        "-w, --width                  Destination images's width\n"
        "-h, --height                 Destination images's height\n"
        "-r, --rotate                 Image rotation degree, the value should be 90, 180 or 270\n"
        "-V, --vflip                  Vertical Mirror\n" 
        "-H, --hflip                  Horizontal Mirror\n"
        "-c, --crop                   Crop, format: x,y,w,h\n"
        "\n",
        argv[0]);
}

static const char *short_options = "i:w:h:r:VHc:";

static struct option long_options[] = {
    {"input", required_argument, NULL, 'i'},
    {"width", required_argument, NULL, 'w'},
    {"height", required_argument, NULL, 'h'},
    {"rotate", required_argument, NULL, 'r'},
    {"vflip", no_argument, NULL, 'V'},
    {"hflip", no_argument, NULL, 'H'},
    {"crop", required_argument, NULL, 'c'},
    {NULL, 0, NULL, 0}
};

static void parse_crop_parameters(char *crop, __u32 *cropx, __u32 *cropy, __u32 *cropw, __u32 *croph)
{
    char *p, *buf;
    const char *delims = ".,";
    __u32 v[4] = {0,0,0,0};
    int i = 0;

    buf = strdup(crop);
    p = strtok(buf, delims);
    while(p != NULL) {
        v[i++] = atoi(p);
        p = strtok(NULL, delims);

        if(i >=4)
             break;
    }

    *cropx = v[0];
    *cropy = v[1];
    *cropw = v[2];
    *croph = v[3];
}

int rga_convert(FILE *in, __u32 w, __u32 h, RgaRotate r, int V, int H, __u32 cx, __u32 cy, __u32 cw, __u32 ch)
{
    int ret = 0;
    rockchip_rga rga;
    Mat mat;
    /* NV12 Format size: w * h * 3 / 2 */
    int size = w * h * 3 / 2;
    int size_align16 = DRM_ALIGN(w, 16) * DRM_ALIGN(h, 16) * 3 / 2;
    void *buf = malloc(size_align16);

    cv::namedWindow(winname);
    cv::moveWindow(winname, w, h);

    /* Todo: allocate buffer for rga destination */
    mat.create(cv::Size(DRM_ALIGN(w, 16), DRM_ALIGN(h, 16)), CV_8UC3);

    ret = rga.init();
    if(ret < 0)
	goto exit;

    while(fread(buf, 1, size, in) == size && !feof(in)) {
	int index;
	size_t len = 0;
	struct rga_format src_fmt, dst_fmt;
	struct rga_crop crop;

	/* TODO: Set source format */
	src_fmt.width = w;
	src_fmt.height = h;
	src_fmt.format = V4L2_PIX_FMT_NV12;

	/* TODO: Set destination format */
	dst_fmt.width = w;
	dst_fmt.height = h;
	dst_fmt.format = V4L2_PIX_FMT_RGB24;

	/* TODO: Set corp */
	crop.x = cx;
	crop.y = cy;
	crop.h = ch;
	crop.w = cw;

	/* TODO: Run and start to convert */
        ret = rga.convert(buf, mat.data, src_fmt, dst_fmt, crop, r);
	if(ret < 0)
	    goto exit;

        imshow(winname, mat);

	cv::waitKey(30);
    }
	
exit:
    free(buf);
    fclose(in);

    return ret;
}

int main(int argc, char **argv)
{
    int c, r;
    char index = -1;
    FILE *in = fopen("test.nv12", "r");

    __u32 width = 640, height = 480;
    RgaRotate rotate = RGA_ROTATE_NONE;
    __u32 cropx = 0, cropy = 0, cropw = 0, croph = 0;
    int vflip = 0, hflip = 0;

    if(in == NULL) {
        fprintf(stderr, "Open file failed\n");
	return -EIO;
    }

    while((c = getopt_long(argc, argv, short_options, long_options, NULL)) != -1) {
        switch (c) {
        case 'w':
            width = atoi(optarg);
            break;
        case 'h':
            height = atoi(optarg);
            break;
        case 'r':
            r = atoi(optarg);
            switch(r) {
                case 0: rotate = RGA_ROTATE_NONE; break;
                case 90: rotate = RGA_ROTATE_90; break;
                case 180: rotate = RGA_ROTATE_180; break;
                case 270: rotate = RGA_ROTATE_270; break;
                default:
                    printf("roate %d is not supported\n", r);
                    return -1; 
            }
            break;
        case 'V':
            vflip = 1;
            break;
        case 'H':
            hflip = 1;
            break;
        case 'c':
            parse_crop_parameters(optarg, &cropx, &cropy, &cropw, &croph);
            break;
        default:
            usage(argv);
            return 0; 
        }
    }

    printf("width = %u, height = %u, rotate = %u, vflip = %d, hflip = %d, crop = [%u, %u, %u, %u]\n",
            width, height, rotate, vflip, hflip, cropx, cropy, cropw, croph);

    if(vflip == 1)
        rotate = RGA_ROTATE_VFLIP;
    else if(hflip == 1)
        rotate = RGA_ROTATE_HFLIP;

    return rga_convert(in, width, height, rotate, vflip, hflip, cropx, cropy, cropw, croph);
}
